// /***************************************************************************
//                           ezmatch  -  description
//                              -------------------
//     begin                : Feb. 18, 2008
//     copyright            : (C) 2008 by Allen
//     email                : bon_ami_@hotmail.com
//  ***************************************************************************/
//
// /***************************************************************************
//  *                                                                         *
//  *   Explicit Distribution Limitation                                      *
//  *   This rule overrides others below.                                     *
//  *   This program may not be modified or used by, or, if possible,         *
//  *   redistributed to people described as below,                           *
//  *   1.Japanese who hold hostility against Chinese.                        *
//  *   2.or, those who discriminate against people based solely on race,     *
//  *     gender or sexual orientation.                                       *
//  *                                                                         *
//  ***************************************************************************/
// /***************************************************************************
//  *                                                                         *
//  *   This program is free software; you can redistribute it and/or modify  *
//  *   it under the terms of the GNU General Public License as published by  *
//  *   the Free Software Foundation; either version 2 of the License, or     *
//  *   (at your option) any later version.                                   *
//  *                                                                         *
//  ***************************************************************************/
// /*
//  *	code of EZ Match Project main functionalities
//  */
//

#include "ezmatch.h"

#define IMAGEFILE <ezmatch/ezmatch.iml>
#include <Draw/iml_source.h>

bool ConfigWin::HotKey(dword key)
{
	switch (key)
	{
		case K_ENTER:
			Config_set();
			return true;
		case K_ESCAPE:
			Config_notset();
			return true;
	}
	return false;
}

ConfigWin::ConfigWin()
{
	Title("configurations");
	Add(lbl_alloc.SetText("forward string").LeftPos(10, 200).TopPos(10, 50));
	Add(lbl_free.SetText("backward string").LeftPos(220, 200).TopPos(10, 50));
	Add(edit_alloc.LeftPos(10, 200).TopPos(30, 20));
	Add(edit_free.LeftPos(220, 200).TopPos(30, 20));
	Add(lbl_head.SetText("criterion beginning string").LeftPos(10, 200).TopPos(70, 50));
	Add(lbl_tail.SetText("criterion ending string").LeftPos(220, 200).TopPos(70, 50));
	Add(edit_head.LeftPos(10, 200).TopPos(90, 20));
	Add(edit_tail.LeftPos(220, 200).TopPos(90, 20));
	Add(but_ok.SetLabel("OK").LeftPos(10, 200).TopPos(130, 30));

	but_ok <<= THISBACK(Config_set);
	Add(but_ng.SetLabel("Cancel").LeftPos(220, 200).TopPos(130, 30));
	SetRect(0, 0, 430, 150);
	but_ng <<= THISBACK(Config_notset);
}

void ConfigWin::Config_set()
{
	Close();
}

void ConfigWin::Config_notset()
{
	edit_alloc.Clear();
	edit_free.Clear();
	edit_head.Clear();
	edit_tail.Clear();
	Close();
}

void ConfigWin::Set(std::string allocs, std::string frees,
	std::string head, std::string tail)
{
	edit_alloc.SetText(allocs);
	edit_free.SetText(frees);
	edit_head.SetText(head);
	edit_tail.SetText(tail);
}

void ConfigWin::Get(std::string &allocs, std::string &frees,
	std::string &head, std::string &tail)
{
	String strw;
	const char *str;

	if (edit_alloc.GetLength())
	{
		strw = edit_alloc;
		str = strw;
		allocs = str;
	}
	if (edit_free.GetLength())
	{
		strw = edit_free;
		str = strw;
		frees = str;
	}
	if (edit_head.GetLength())
	{
		strw = edit_head;
		str = strw;
		head = str;
	}
	if (edit_tail.GetLength())
	{
		strw = edit_tail;
		str = strw;
		tail = str;
	}
}

bool ezmatch::HotKey(dword key)
{
	switch (key)
	{
		case K_ENTER:
			Do_buttPush();
			return true;
	}
	return TopWindow::HotKey(key);
}

ezmatch::ezmatch()
{
	max_edit.SetData(60000);
	direct_swit.Set(0, 0, "forward");
	direct_swit.Set(1, 1, "reverse");
	direct_swit.EnableCase(0);
	file_edit.SetData("log.txt");
	brs.Type("log files", "*.log");
	brs.Type("text files", "*.txt");
	brs.AllFilesType();
		proc_prog.Hide();
	Zoomable();
	Sizeable();
	CtrlLayout(*this, "EZ Match");
	AddFrame(menu);
	Icon(IMAGECLASS::Get("icon"), IMAGECLASS::Get("icon"));

	file_butt <<= THISBACK(File_butt);
	direct_swit <<= THISBACK(Direct_swit);
	do_butt.WhenPush = THISBACK(Do_buttPush);
	WhenClose = THISBACK(WClose);
	menu.Set(THISBACK(Menu));

	config_alloc = "INITMALLOC";
	config_free = "FREEMALLOC";
	config_head = "addr (";
	config_tail = ")";
}

void ezmatch::Help()
{
	PromptOK("{{1"
		"@l [A4@k= EZ Match 1.1]"
		"::"
		" [2<   This program pairs lines containing INITMALLOC and FREEMALLOC with same string "
			"\"addr (...)\" in a file and shows the orphan lines "
			"(line content prefixed by line number, "
			"which is not exact and manual searching is strongly recommended). "
			"The strings can be configured and the comparing order can be switched. "
			"Thus this program is not limited to be used for memory leakage statistics, "
			"but for more general line pairing. The shortage is the pairing does not "
			"take lines' orders into consideration. "
			"So successive INITMALLOC or FREEMALLOC lines for same criterion string cause "
			"inaccurate results. "
			"&  Buffer size is the approximate length each time the file is processed."
			"&     Note that operations include opening a file and computing, which may be time consuming!]"
		"}}");
}

void ezmatch::About()
{
	PromptOK("{{1"
		"@l [A4@k= EZ Match 1.1]"
		":: [2< "
		"[= Explicit Distribution Limitation]"
		"&  This rule overrides others below."
		"&    This program may not be modified or used by, or, if possible, "
		"redistributed to people described as below,"
		"& [o Japanese who hold hostility against Chinese.]"
		"& [o or, those who discriminate against people based solely on race, gender or sexual orientation.]]"
		":: [2 "
		"  This program is free software; you can redistribute it and/or modify "
		"it under the terms of the GNU General Public License as published by "
		"the Free Software Foundation; either version 2 of the License, or "
		"(at your option) any later version.]"
		"::"
		"@g [@Y> Integreated by Allen. Technology (example) by Guo Yongping."
			"&web links:   [^http://ezproject.sourceforge.net^ EZ Project] "
			"   [^mailto://bon_ami_@hotmail.com^ E-Mail Me]]"
		"::"
		"@K [@L> using [^http://www.ultimatepp.org^ Ultimate`+`+] for Windows]"
		"}}");
}

void ezmatch::HelpMenu(Bar& bar)
{
	bar.Add(t_("&Help..."), THISBACK(Help));
	bar.Add(t_("&About..."), THISBACK(About));
}

void ezmatch::SaveAll()
{
	int loop = 0;
	
	do
	{
		if (brs.ExecuteSaveAs("select the file to be saved to"))
		{
			FileOut fo(brs.Get());
			if (!fo.IsOpen())
				loop = PromptRetryCancel("File cannot be opened for saving!");
			else
			{
				text_edit.Save(fo);
			}
		}
	} while (loop);
}

void ezmatch::AppendAll()
{
	int loop = 0;
	
	do
	{
		if (brs.ExecuteSaveAs("select the file to be appended to"))
		{
			FileAppend fo(brs.Get());
			if (!fo.IsOpen())
				loop = PromptRetryCancel("File cannot be opened for saving!");
			else
			{
				text_edit.Save(fo);
			}
		}
	} while (loop);
}

void ezmatch::CopyAll()
{
	text_edit.SelectAll();
	text_edit.Copy();
}

void ezmatch::ClearAll()
{
	text_edit.Clear();
	text_alloc.Clear();
	text_free.Clear();
	proc_prog.Hide();
	proc_lbl.Hide();
}

void ezmatch::EditMenu(Bar& bar)
{
	bar.Add(t_("&Copy all"), THISBACK(CopyAll));
	bar.Add(t_("&Clear all"), THISBACK(ClearAll));
	bar.Add(t_("&Save current output to..."), THISBACK(SaveAll));
	bar.Add(t_("&Append current output to..."), THISBACK(AppendAll));
}

void ezmatch::FileMenu(Bar& bar)
{
	bar.Add(t_("&Open..."), THISBACK(File_butt)).Key(K_CTRL_O);
	bar.Add(t_("&Config"), THISBACK(Config));
	bar.Add(t_("&Run"), THISBACK(Do_buttPush));
	bar.Separator();
	bar.Add(t_("&Exit"), THISBACK(WClose));
}

void ezmatch::Menu(Bar& bar)
{
	menu.Add(t_("&File"), THISBACK(FileMenu));
	edit_menu = &(menu.Add(t_("&Edit"), THISBACK(EditMenu)).Enable(false));
	menu.GapRight();
	menu.Add(t_("&Help"), THISBACK(HelpMenu));
}

void ezmatch::Config(void)
{
	config.Set(config_alloc, config_free, config_head, config_tail);
	config.Execute();
	config.Get(config_alloc, config_free, config_head, config_tail);
}

void ezmatch::Layout()
{
	static Rect prev_size;
	Rect curr_size = GetRect();
	if (curr_size != prev_size)
	{
		Rect ctrl_size = text_edit.GetRect();
		Rect menu_size = menu.GetRect();
		text_edit.SetRect(ctrl_size.left, ctrl_size.top,
			curr_size.GetWidth(), curr_size.GetHeight() - menu_size.Height() - ctrl_size.top);
		prev_size = curr_size;
	}
}

void ezmatch::File_butt()
{
	brs.ExecuteOpen("select the file to be processed");
	String brs_str = brs.Get();
	if (!brs_str.IsEmpty())
		file_edit.SetData(brs_str);
	proc_prog.Hide();
}

void ezmatch::Direct_swit()
{
	proc_prog.Hide();
	switch (direct_swit)
	{
		case 0:
			text_edit.SetData(text_alloc);
			break;
		case 1:
			text_edit.SetData(text_free);
	}
}

int memchk_main(std::string filename, std::pair<std::string, std::string>,
	std::pair<std::string, std::string>);
int mem_check_result(int &max_len, std::string &res);
void mem_check_clear(void);

void ezmatch::WClose()
{
	if (do_butt.GetTip().Find("continue") >= 0)
		mem_check_clear();
	Close();
}

void ezmatch::Do_buttPush()
{
	static bool opened = false;
	static int line_num = 0;
	static int prev_num = 0;
	int done;
	int more = max_edit.GetData();

	if (!file_edit.GetLength())
		return;

	do_butt.Disable();
	edit_menu->Enable(false);
	if (more < 0)
		more = 0;
	if (!opened)
	{
		const char *filename;
		String filenamefull = file_edit.GetText().ToString();
		int pos = 0;

		/* convert single back slashes to double ones */
		while ((pos = filenamefull.Find('\\', pos)) >= 0)
			if (filenamefull[++pos] != '\\')
				filenamefull.Insert(pos++, '\\');
		filename = filenamefull;
		Refresh();
		Sync();
		line_num = memchk_main(filename,
			(0 == direct_swit) ? std::make_pair(config_alloc, config_free)
				: std::make_pair(config_free, config_alloc),
				std::make_pair(config_head, config_tail));
		if (line_num < 0)
		{
			String msg("File Reading Failure!");
			text_edit.Set(msg);
			do_butt.Enable();
			edit_menu->Enable();
			return;
		}
		opened = true;
		more = -more;
	}

	std::string res;
	done = mem_check_result(more, res);
	text_edit.Set(res);

	if (done < more)
	{
		do_butt.SetLabel("continue");
		do_butt.Tip("continue processing");
		direct_swit.Disable();
	}
	else
	{
		if (res.empty())
			text_edit.Set("No Result.", CHARSET_DEFAULT);
		do_butt.SetLabel("begin");
		do_butt.Tip("start processing");
		mem_check_clear();
		if (0 == direct_swit)
			text_alloc = text_edit.GetData();
		else
			text_free = text_edit.GetData();
		direct_swit.Enable();
		more = done;
		opened = false;
	}
	if (!more)
		proc_prog.Hide();
	else
	{
		proc_prog.Set(done, more);
		proc_prog.Show();
	}
	char buf[20];
	snprintf(buf, sizeof(buf), "%d/%d", done, more);
	proc_lbl = buf;
	proc_lbl.Show();
	do_butt.Enable();
	edit_menu->Enable();
}

GUI_APP_MAIN
{
	ezmatch bj;
	
	bj.Run();
}

